@@ -3,8 +3,18 @@ package HTML::Tiny;
 use strict;
 use Carp;
 
+=head1 NAME
+
+HTML::Tiny - Lightweight, dependency free HTML/XML generation
+
+=head1 VERSION
+
+This document describes HTML::Tiny version 0.903
+
+=cut
+
 use vars qw/$VERSION/;
-$VERSION = '0.902';
+$VERSION = '0.903';
 
 BEGIN {
 
@@ -34,217 +44,6 @@ my %DEFAULT_AUTO = (
     method => 'tag'
 );
 
-my %ENT_MAP = (
-    '&' => '&amp;',
-    '<' => '&lt;',
-    '>' => '&gt;',
-    '"' => '&quot;',
-    "'" => '&apos;',
-);
-
-my @UNPRINTABLE = qw(
-  z    x01  x02  x03  x04  x05  x06  a
-  x08  t    n    v    f    r    x0e  x0f
-  x10  x11  x12  x13  x14  x15  x16  x17
-  x18  x19  x1a  e    x1c  x1d  x1e  x1f
-);
-
-sub new {
-    my $self = bless {}, shift;
-    $self->_set_auto( 'method', 'closed', @DEFAULT_CLOSED );
-    $self->_set_auto( 'suffix', "\n",     @DEFAULT_NEWLINE );
-    return $self;
-}
-
-sub _set_auto {
-    my ( $self, $kind, $value ) = @_;
-
-    if ( defined $value ) {
-        for ( @_ ) {
-            $self->{autotag}->{$kind}->{$_} = $value;
-        }
-    }
-    else {
-        delete @{ $self->{autotag}->{$kind} }{@_};
-    }
-}
-
-sub _str {
-    my $obj = shift;
-    if ( my $ref = ref $obj ) {
-
-        # Flatten array refs...
-        return join '', @$obj
-          if 'ARRAY' eq $ref;
-
-        # ...stringify objects...
-        my $str = eval { $obj->as_string };
-        return $str unless $@;
-    }
-
-    # ...default stringification
-    return "$obj";
-}
-
-# URL encode a string
-sub url_encode {
-    my $str = _str( $_[1] );
-    $str =~ s/([^A-Za-z0-9_])/$1 eq ' ' ? '+' : sprintf("%%%02x", ord($1))/eg;
-    return $str;
-}
-
-sub url_decode {
-    my $str = $_[1];
-    $str =~ s/[+]/ /g;
-    $str =~ s/%([0-9a-f]{2})/chr(hex($1))/eg;
-    return $str;
-}
-
-# Turn a hash reference into a query string.
-sub query_encode {
-    my $self = shift;
-    my $hash = shift || {};
-    return join '&', map {
-        join( '=', map { $self->url_encode( $_ ) } ( $_, $hash->{$_} ) )
-    } sort grep { defined $hash->{$_} } keys %$hash;
-}
-
-# (X)HTML entity encode a string
-sub entity_encode {
-    my $str = _str( $_[1] );
-    $str =~ s/([<>&'"])/$ENT_MAP{$1}/eg;
-    return $str;
-}
-
-sub _tag_value {
-    my ( $self, $val ) = @_;
-    return '' if ref $val;
-    return '="' . $self->entity_encode( $val ) . '"';
-}
-
-sub validate_tag {
-
-    # Do nothing. Subclass to throw an error for invalid tags
-}
-
-sub _tag {
-    my $self   = shift;
-    my $closed = shift;
-    my $name   = shift;
-
-    croak "Attributes must be passed as hash references"
-      if grep { 'HASH' ne ref $_ } @_;
-
-    # Merge attribute hashes
-    my %attr = map { %$_ } @_;
-
-    $self->validate_tag( $closed, $name, \%attr );
-
-    # Generate markup
-    return "<$name"
-      . join( '',
-        map         { ' ' . $_ . $self->_tag_value( $attr{$_} ) }
-          sort grep { defined $attr{$_} } keys %attr )
-      . ( $closed ? ' />' : '>' );
-}
-
-sub tag {
-    my $self = shift;
-    my $name = shift;
-
-    my %attr = ();
-    my @out  = ();
-
-    for my $a ( @_ ) {
-        if ( 'HASH' eq ref $a ) {
-
-            # Merge into attributes
-            %attr = ( %attr, %$a );
-        }
-        else {
-
-            # Generate markup
-            push @out,
-              $self->_tag( 0, $name, \%attr )
-              . _str( $a )
-              . $self->close( $name );
-        }
-    }
-
-    # Special case: generate an empty tag pair if there's no content
-    push @out, $self->_tag( 0, $name, \%attr ) . $self->close( $name )
-      unless @out;
-
-    return wantarray ? @out : join '', @out;
-}
-
-sub open   { shift->_tag( 0, @_ ) }
-sub closed { shift->_tag( 1, @_ ) }
-
-# Generate a closing (X)HTML tag
-sub close { "</$_[1]>" }
-
-sub auto_tag {
-    my $self = shift;
-    my $name = shift;
-    my ( $method, $post )
-      = map { $self->{autotag}->{$_}->{$name} || $DEFAULT_AUTO{$_} }
-      ( 'method', 'suffix' );
-    my @out = map { $_ . $post } $self->$method( $name, @_ );
-    return wantarray ? @out : join '', @out;
-}
-
-# Minimal JSON encoder. Provided here for completeness - it's useful when generating JS.
-sub _json_encode {
-    my ( $self, $seen, $obj ) = @_;
-
-    return 'null' unless defined $obj;
-
-    if ( my $type = ref $obj ) {
-        croak "json_encode can't handle self referential structures"
-          if $seen->{$obj}++;
-        my $rep = ( 'HASH' eq $type )
-          ? (
-            '{' . join(
-                ',',
-                map {
-                        $self->_json_encode( $seen, $_ ) . ':'
-                      . $self->_json_encode( $seen, $obj->{$_} )
-                  } sort keys %$obj
-              )
-              . '}'
-          )
-          : ( 'ARRAY' eq $type ) ? ( '['
-              . join( ',', map { $self->_json_encode( $seen, $_ ) } @$obj )
-              . ']' )
-          : undef;
-        delete $seen->{$obj};
-        return $rep if defined $rep;
-    }
-
-    return $obj if $obj =~ /^-?\d+(?:[.]\d+)?$/;
-
-    $obj = _str( $obj );
-    $obj =~ s/\\/\\\\/g;
-    $obj =~ s/"/\\"/g;
-    $obj =~ s/ ( [\x00-\x1f] ) / '\\' . $UNPRINTABLE[ ord($1) ] /gex;
-
-    return qq{"$obj"};
-}
-
-sub json_encode { shift->_json_encode( {}, @_ ) }
-
-1;
-__END__
-
-=head1 NAME
-
-HTML::Tiny - Lightweight, dependency free HTML/XML generation
-
-=head1 VERSION
-
-This document describes HTML::Tiny version 0.902
-
 =head1 SYNOPSIS
 
     use HTML::Tiny;
@@ -307,6 +106,20 @@ Create a new C<< HTML::Tiny >>. No arguments
 
 =back
 
+=cut
+
+sub new {
+    my $self = bless {}, shift;
+    $self->_set_auto( 'method', 'closed', @DEFAULT_CLOSED );
+    $self->_set_auto( 'suffix', "\n",     @DEFAULT_NEWLINE );
+    return $self;
+}
+
+sub _set_auto {
+    my ( $self, $kind, $value ) = @_;
+    $self->{autotag}->{$kind}->{$_} = $value for @_;
+}
+
 =head2 HTML Generation
 
 =over
@@ -442,6 +255,64 @@ Note how you don't need a td() for every cell or a tr() for every row.
 Notice also how the square brackets around the rows prevent tr() from
 wrapping each individual cell.
 
+Often when generating nested HTML you will find yourself writing
+corresponding nested calls to HTML generation methods. The table
+generation code above is an example of this.
+
+If you prefer these nested method calls can be deferred like this:
+
+    print $h->table(
+        [
+            \'tr',
+            [ \'th', 'Name',     'Score', 'Position' ],
+            [ \'td', 'Therese',  90,      1 ],
+            [ \'td', 'Chrissie', 85,      2 ],
+            [ \'td', 'Andy',     50,      3 ]
+        ]
+    );
+
+In general a nested call like
+
+    $h->method( args )
+
+may be rewritten like this
+
+    [ \'method', args ]
+
+This allows complex HTML to be expressed as a pure data structure. See
+the C<stringify> method for more information.
+
+=cut
+
+sub tag {
+    my ( $self, $name ) = splice @_, 0, 2;
+
+    my %attr = ();
+    my @out  = ();
+
+    for my $a ( @_ ) {
+        if ( 'HASH' eq ref $a ) {
+
+            # Merge into attributes
+            %attr = ( %attr, %$a );
+        }
+        else {
+
+            # Generate markup
+            push @out,
+              $self->_tag( 0, $name, \%attr )
+              . $self->stringify( $a )
+              . $self->close( $name );
+        }
+    }
+
+    # Special case: generate an empty tag pair if there's no content
+    push @out, $self->_tag( 0, $name, \%attr ) . $self->close( $name )
+      unless @out;
+
+    return wantarray ? @out : join '', @out;
+}
+
 =item C<< open( $name, ... ) >>
 
 Generate an opening HTML or XML tag. For example:
@@ -464,6 +335,10 @@ As for C<< tag >> multiple attribute hash references will be merged. The example
 
     print $h->open('marker', { lat => 57.0 }, { lon => -2 });
 
+=cut
+
+sub open { shift->_tag( 0, @_ ) }
+
 =item C<< close( $name ) >>
 
 Generate a closing HTML or XML tag. For example:
@@ -474,6 +349,10 @@ would print:
 
     </marker>
 
+=cut
+
+sub close { "</$_[1]>" }
+
 =item C<< closed( $name, ... ) >>
 
 Generate a closed HTML or XML tag. For example
@@ -492,11 +371,92 @@ would print:
 
     <marker lat="57.0" lon="-2" />
 
+=cut
+
+sub closed { shift->_tag( 1, @_ ) }
+
 =item C<< auto_tag( $name, ... ) >>
 
 Calls either C<< tag >> or C<< closed >> based on built in rules
 for the tag. Used internally to implement the tag-named methods.
 
+=cut
+
+sub auto_tag {
+    my ( $self, $name ) = splice @_, 0, 2;
+    my ( $method, $post )
+      = map { $self->{autotag}->{$_}->{$name} || $DEFAULT_AUTO{$_} }
+      ( 'method', 'suffix' );
+    my @out = map { $_ . $post } $self->$method( $name, @_ );
+    return wantarray ? @out : join '', @out;
+}
+
+=item C<< stringify( $obj ) >>
+
+Called internally to obtain string representations of values.
+
+It also implements the deferred method call notation (mentioned above) so that
+
+    my $table = $h->table(
+        [
+            $h->tr(
+                [ $h->th( 'Name', 'Score', 'Position' ) ],
+                [ $h->td( 'Therese',  90, 1 ) ],
+                [ $h->td( 'Chrissie', 85, 2 ) ],
+                [ $h->td( 'Andy',     50, 3 ) ]
+            )
+        ]
+    );
+
+may also be written like this:
+
+    my $table = $h->stringify(
+        [
+            \'table',
+            [
+                \'tr',
+                [ \'th', 'Name',     'Score', 'Position' ],
+                [ \'td', 'Therese',  90,      1 ],
+                [ \'td', 'Chrissie', 85,      2 ],
+                [ \'td', 'Andy',     50,      3 ]
+            ]
+        ]
+    );
+
+Any reference to an array whose first element is a reference to a scalar
+
+    [ \'methodname', args ]
+
+is executed as a call to the named method with the specified args.
+
+=cut
+
+sub stringify {
+    my ( $self, $obj ) = @_;
+    if ( my $ref = ref $obj ) {
+
+        # Flatten array refs...
+        if ( 'ARRAY' eq $ref ) {
+            # Check for deferred method call specified as a scalar
+            # ref...
+            if ( @$obj && 'SCALAR' eq ref $obj->[0] ) {
+                my ( $method, @args ) = @$obj;
+                return join '', $self->$$method( @args );
+            }
+            else {
+                return join '', map { $self->stringify( $_ ) } @$obj;
+            }
+        }
+
+        # ...stringify objects...
+        my $str = eval { $obj->as_string };
+        return $str unless $@;
+    }
+
+    # ...default stringification
+    return "$obj";
+}
+
 =back
 
 =head2 Methods named after tags
@@ -551,6 +511,13 @@ URL encode a string. Spaces become '+' and unprintable characters are
 encoded as '%' + their hexadecimal character code.
 
     $h->url_encode( ' <hello> ' )   # returns '+%3chello%3e+'
+=cut
+
+sub url_encode {
+    my $str = $_[0]->stringify( $_[1] );
+    $str =~ s/([^A-Za-z0-9_])/$1 eq ' ' ? '+' : sprintf("%%%02x", ord($1))/eg;
+    return $str;
+}
 
 =item C<< url_decode( $str ) >>
 
@@ -558,6 +525,15 @@ URL decode a string. Reverses the effect of C<< url_encode >>.
 
     $h->url_decode( '+%3chello%3e+' )   # returns ' <hello> '
 
+=cut
+
+sub url_decode {
+    my $str = $_[1];
+    $str =~ s/[+]/ /g;
+    $str =~ s/%([0-9a-f]{2})/chr(hex($1))/eg;
+    return $str;
+}
+
 =item C<< query_encode( $hash_ref ) >>
 
 Generate a query string from an anonymous hash of key, value pairs:
@@ -568,6 +544,16 @@ would print
 
     a=1&b=2
 
+=cut
+
+sub query_encode {
+    my $self = shift;
+    my $hash = shift || {};
+    return join '&', map {
+        join( '=', map { $self->url_encode( $_ ) } ( $_, $hash->{$_} ) )
+    } sort grep { defined $hash->{$_} } keys %$hash;
+}
+
 =item C<< entity_encode( $str ) >>
 
 Encode the characters '<', '>', '&', '\'' and '"' as their HTML entity
@@ -579,6 +565,102 @@ would print:
 
     &lt;&gt;&apos;&quot;&amp;
 
+=cut
+
+{
+    my %ENT_MAP = (
+        '&' => '&amp;',
+        '<' => '&lt;',
+        '>' => '&gt;',
+        '"' => '&quot;',
+        "'" => '&apos;',
+    );
+
+    sub entity_encode {
+        my $str = $_[0]->stringify( $_[1] );
+        $str =~ s/([<>&'"])/$ENT_MAP{$1}/eg;
+        return $str;
+    }
+}
+
+sub _tag_value {
+    my ( $self, $val ) = @_;
+    return '' if ref $val;
+    return '="' . $self->entity_encode( $val ) . '"';
+}
+
+sub validate_tag {
+
+    # Do nothing. Subclass to throw an error for invalid tags
+}
+
+sub _tag {
+    my ( $self, $closed, $name ) = splice @_, 0, 3;
+
+    croak "Attributes must be passed as hash references"
+      if grep { 'HASH' ne ref $_ } @_;
+
+    # Merge attribute hashes
+    my %attr = map { %$_ } @_;
+
+    $self->validate_tag( $closed, $name, \%attr );
+
+    # Generate markup
+    return "<$name"
+      . join( '',
+        map         { ' ' . $_ . $self->_tag_value( $attr{$_} ) }
+          sort grep { defined $attr{$_} } keys %attr )
+      . ( $closed ? ' />' : '>' );
+}
+
+{
+    my @UNPRINTABLE = qw(
+      z    x01  x02  x03  x04  x05  x06  a
+      x08  t    n    v    f    r    x0e  x0f
+      x10  x11  x12  x13  x14  x15  x16  x17
+      x18  x19  x1a  e    x1c  x1d  x1e  x1f
+    );
+
+    # Minimal JSON encoder. Provided here for completeness - it's useful
+    # when generating JS.
+    sub _json_encode {
+        my ( $self, $seen, $obj ) = @_;
+
+        return 'null' unless defined $obj;
+
+        if ( my $type = ref $obj ) {
+            croak "json_encode can't handle self referential structures"
+              if $seen->{$obj}++;
+            my $rep = ( 'HASH' eq $type )
+              ? (
+                '{' . join(
+                    ',',
+                    map {
+                            $self->_json_encode( $seen, $_ ) . ':'
+                          . $self->_json_encode( $seen, $obj->{$_} )
+                      } sort keys %$obj
+                  )
+                  . '}'
+              )
+              : ( 'ARRAY' eq $type ) ? ( '['
+                  . join( ',', map { $self->_json_encode( $seen, $_ ) } @$obj )
+                  . ']' )
+              : undef;
+            delete $seen->{$obj};
+            return $rep if defined $rep;
+        }
+
+        return $obj if $obj =~ /^-?\d+(?:[.]\d+)?$/;
+
+        $obj = $self->stringify( $obj );
+        $obj =~ s/\\/\\\\/g;
+        $obj =~ s/"/\\"/g;
+        $obj =~ s/ ( [\x00-\x1f] ) / '\\' . $UNPRINTABLE[ ord($1) ] /gex;
+
+        return qq{"$obj"};
+    }
+}
+
 =item C<< json_encode >>
 
 Encode a data structure in JSON (Javascript) format:
@@ -606,6 +688,13 @@ Because JSON is valid Javascript this method can be useful when generating ad-ho
     # var someVar = {"history":[32,37,41,45],"name":"Fred","score":45};
     # </script>
 
+=cut
+
+sub json_encode { shift->_json_encode( {}, @_ ) }
+
+1;
+__END__
+
 =back
 
 =head2 Subclassing
